/*  
    Roaminfo - RoamAbout Access Point Information
    Copyright (C) 2000 Greg Hankins <gregh@twoguys.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA 
*/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "snmp.h"
#include "roaminfo.h"

char hostname[STRING255];    /* hostname of remote device */
char community[STRING255];   /* read community string */
char *request[MAXQUERY];     /* SNMP requests */
char *response[MAXQUERY];    /* SNMP responses */
char *buf;                   /* tmp */

extern struct snmp_session session;

/*
 * Main.
 */
int main(int argc, char *argv[]) {

    int i;                               /* tmp */
    char c;                              /* tmp */
    int system = 0;                      /* true if we need to show system */
    int hardware = 0;                    /* true if we need to show hardware */
    int stats = 0;                       /* true if we need to show stats */
    extern char *optarg;                 /* getopt */
    extern int optind, opterr, optopt;   /* getopt */

    /* getopt */
    while ((c = getopt(argc, argv, "?isawhl")) != EOF) {
        switch (c) {
            case 'a': {
                system = 1;
                hardware = 1;
                stats = 1;
                break;
            }
            case 'i': {
                system = 1;
                break;
            }
            case 's': {
                stats = 1;
                break;
            }
            case 'w': {
                hardware = 1;
                break;
            }
            case 'l': {
                show_license(argv[0]);
                exit(1);
                break;
            }
            case 'h':
            default: {
                usage(argv[0]);
                exit(1);
                break;
            }
        }
    }

    /* check arguments */
    if (argc < 3) {
        usage(argv[0]);
        exit(1);
    }

    /* initialize stuff */
    for (i = 0; i < MAXQUERY; i++) {
        request[i] = (char*)malloc(STRING128);
        if (request[i] == NULL) {
            fprintf(stderr, "malloc() failed\n");
            exit(-1);
        }        

        response[i] = (char*)malloc(STRING128);
        if (request[i] == NULL) {
            fprintf(stderr, "malloc() failed\n");
            exit(-1);
        }        
    }

    strncpy(hostname, argv[argc - 2], sizeof(hostname) - 1);
    strncpy(community, argv[argc - 1], sizeof(community) - 1);

    snmp_init(hostname, community);

    /* do something by default */
    if (system + hardware + stats == 0) {
        system = 1;
    }

    /* make sure we are talking to the right hardware */
    check_hardware();

    /* run queries */
    if (system == 1) show_system();
    if (hardware == 1) show_hardware();
    if (stats == 1) show_stats();

    /* cleanup */
    for (i = 0; i < MAXQUERY; i++) {
        free(response[i]);
        free(request[i]);
    }

    exit(0);
}

/*
 * Show system information.
 */
void check_hardware() {

    strcpy(request[0], SNMP_sysObjectID_0);

    if (snmp_get(request, response, 1) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    /* check to make sure this is the right box */
    if (strncmp(response[0] + 5, ROAMABOUT, ROAMABOUTLENGTH) != 0) {
        fprintf(stderr, "\nsystem.sysObjectID.0 (%s) does not match.\nAre you sure this device is a RoamAbout?\n", response[0] + 5);
        exit(0);
    }
}

/*
 * Show system information.
 */
void show_system() {

    strcpy(request[0], SNMP_sysDescr_0);
    strcpy(request[1], SNMP_sysObjectID_0);
    strcpy(request[2], SNMP_sysUpTime_0);
    strcpy(request[3], SNMP_sysContact_0);
    strcpy(request[4], SNMP_sysName_0);
    strcpy(request[5], SNMP_sysLocation_0);

    /* request system information */
    if (snmp_get(request, response, 6) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    printf("\n    System Information\n");
    printf("Description: %s\n", response[0]);
/*    printf("ObjectID   : %s\n", response[1] + 5);*/

    /* fix uptime string */
    buf = strstr(response[2], ")");
    buf += 2;
    buf[strlen(buf) - 3] = '\0';
    printf("Uptime     : %s\n", buf);

    if(strlen(response[3])) {
        printf("Contact    : %s\n", response[3]);
    }

    if(strlen(response[4])) {
        printf("Name       : %s\n", response[4]);
    }

    if(strlen(response[5])) {
        printf("Location   : %s\n", response[5]);
    }
}

/*
 * Show hardware information.
 */
void show_hardware() {

    strcpy(request[0], SNMP_sysDescr_0);

    if (snmp_get(request, response, 1) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    /* chop revision string  */
    buf = strstr(response[0], ":");
    buf += 3;
    printf("\n    Hardware Information\n");
    printf("Access Point Revisions: %s\n", buf);

    strcpy(request[0], SNMP_AP_Memory);

    if (snmp_get(request, response, 1) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }
    printf("Access Point Memory   : %d MB\n", (atoi(response[0]) / 1048576));

    strcpy(request[0], SNMP_PC_Card_Versions);

    if (snmp_get(request, response, 1) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    /* fix quotes that appear around string */
    buf = response[0];
    buf++;
    buf[strlen(response[0]) - 2] = '\0';
    printf("PC Card Versions      : %s\n", buf);

    strcpy(request[0], SNMP_ifPhysAddress_1);
    strcpy(request[1], SNMP_ifPhysAddress_2);

    if (snmp_get(request, response, 2) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    printf("Wired MAC Address     : 0%s\n", response[0]);
    printf("Wireless MAC Address  : 0%s\n", response[1]);
}

/*
 * Show statistics.
 */
void show_stats() {

    strcpy(request[0], SNMP_ifInOctets_1);
    strcpy(request[1], SNMP_ifInOctets_2);
    strcpy(request[2], SNMP_ifOutOctets_1);
    strcpy(request[3], SNMP_ifOutOctets_2);
    strcpy(request[4], SNMP_ifInErrors_1);
    strcpy(request[5], SNMP_ifInErrors_2);
    strcpy(request[6], SNMP_ifOutErrors_1);
    strcpy(request[7], SNMP_ifOutErrors_2);

    if (snmp_get(request, response, 8) > 0) {
        fprintf(stderr, "SNMP error\n");
        exit(-1);
    }

    printf("\n    Statistics\n");
    printf("Wired Port    Transmitted    : %s octets\n", response[2]);
    printf("              Received       : %s octets\n", response[0]);
    printf("              Transmit Errors: %s %s\n", response[6], (atoi(response[6]) == 1) ? "packet" : "packets");
    printf("              Receive Errors : %s %s\n", response[4], (atoi(response[4]) == 1) ? "packet" : "packets");


    printf("Wireless Port Transmitted    : %s octets\n", response[3]);
    printf("              Received       : %s octets\n", response[1]);
    printf("              Transmit Errors: %s %s\n", response[7], (atoi(response[7]) == 1) ? "packet" : "packets");
    printf("              Receive Errors : %s %s\n", response[5], (atoi(response[5]) == 1) ? "packet" : "packets");

}

/*
 * Show usage.
 */
void usage(char *s) {

    printf("  Roaminfo v%s - RoamAbout Access Point Information\n", VERSION);
    printf("  Copyright (C) 2000 Greg Hankins\n");
    printf("  Roaminfo comes with ABSOLUTELY NO WARRANTY; for details type `%s -l'.\n", s);
    printf("  Usage: %s [options] <hostname> <community>\n", s);
    printf("  Options:\n");
    printf("    -a   show all\n");
    printf("    -i   show system information\n");
    printf("    -l   show license\n");
    printf("    -s   show statistics\n");
    printf("    -w   show hardware information\n");
}

/*
 * Show the license.
 */
void show_license(char *s) {

    printf("  Roaminfo v%s - RoamAbout Access Point Information\n", VERSION);
    printf("  Copyright (C) 2000 Greg Hankins\n\n");
    printf("  This program is free software; you can redistribute it and/or modify\n  it under the terms of the GNU General Public License as published by\n  the Free Software Foundation; either version 2 of the License, or\n  (at your option) any later version.\n\n");
    printf("  This program is distributed in the hope that it will be useful,\n  but WITHOUT ANY WARRANTY; without even the implied warranty of\n  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n  GNU General Public License for more details.\n\n");
    printf("  You should have received a copy of the GNU General Public License\n  along with this program; if not, write to the Free Software\n  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA\n\n");
}
